Package: gamair
Version: 1.0-2
Author: Simon Wood <simon.wood@r-project.org>
Maintainer: Simon Wood <simon.wood@r-project.org>
Title: Data for 'GAMs: An Introduction with R'
Description: Data sets and scripts used in the book 'Generalized Additive 
             Models: An Introduction with R', Wood (2006,2017) CRC.
Depends: R (>= 2.10)
Suggests: mgcv, lattice, MASS, nlme, lme4, geoR, survival
License: GPL (>= 2)
NeedsCompilation: no
Packaged: 2019-08-23 12:14:27 UTC; sw283
Repository: CRAN
Date/Publication: 2019-08-23 12:40:02 UTC
Built: R 4.4.0; ; 2024-05-30 02:48:04 UTC; unix
